/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32	/* Windows uses _read from io.h */
#include <io.h>
#undef read
#define read _read
#else
#include <unistd.h>
#endif
#include "glsd.h"

GLSDVARS_S *glsdv;
GLSDMETHOD_S *glsd_methods = NULL;	/* known methods */

char **parse_list(char *list, char separator, int *total);


char **
parse_list(char *list, char separator, int *total)
{
  int i;
  char *p, *q, **rv;

  for(i=strlen(list)-1; i >= 0 && list[i] == ' '; i--)
     list[i] = '\0';

  for(i = 0, p = list; p && *p && (q = strchr(p, separator)) != NULL; i++)
     for(p = ++q; *p == ' '; p++);

  rv = (char **) fs_get((i+2)*sizeof(char *));
  memset(rv, 0, (i+2)*sizeof(char *));

  for(i = 0, p = list; p && *p && (q = strchr(p, separator)) != NULL; i++){
     *q = '\0';
     rv[i] = cpystr(p);
     *q = separator;
     for(p = ++q; *p == ' '; p++);
  }
  rv[i] = cpystr(p);
  if(total)
    *total = i+1;
  return rv;
}

int
print_error(GLSDV_S *vp, char *err_msg, int check_type)
{
   int err = 0, i, total;
   double d, *dp;
   int    r, *rp;
   char *p, **list;
   RESULT_S *result;

   switch(check_type){
     case DOUBLE_TYPE:  p = vp->name;
			d = strtod(p, &p);
			if(p && *p)
			 err++;
			else{
			  dp = (double *)fs_get(sizeof(double));
			  result = (RESULT_S *) fs_get(sizeof(RESULT_S));
			  *dp = d;
			  result->result = (void *) dp;
			  vp->result = (void *) result;
			}
			break;

     case INTEGER_TYPE: p = vp->name;
			r = strtol(p, &p, 10);
			if(p && *p)
			 err++;
			else{
			  rp = (int *)fs_get(sizeof(int));
			  result = (RESULT_S *) fs_get(sizeof(RESULT_S));
			  *rp = r;
			  result->result = (void *) rp;
			  vp->result = (void *) result;
			}
			break;

     case DOUBLE_LIST_TYPE: list = parse_list(vp->name, ' ', &total);
			    for(i = 0; (p = list[i]) != NULL; i++){
				d = strtod(p, &p);
				if(p && *p){
				   fprintf(stderr,"Error at %s\n", list[i]);
				   err++;
				   break;
				}
			    }
			    if(err == 0){
			     result = (RESULT_S *) fs_get(sizeof(RESULT_S));
			     dp = (double *) fs_get((total + 1)*sizeof(double));
			     for(i = 0; (p = list[i]) != NULL; i++)
				dp[i] = strtod(p, &p);
			      dp[i] = 0.0;
			      result->result = (void *) dp;
			      result->total  = total;
			      vp->result = (void *)result;
			    }
			    for(i = 0; list[i] != NULL; i++)
				fs_give((void **)&list[i]);
			     fs_give((void **)list);
			break;

     case INTEGER_LIST_TYPE: list = parse_list(vp->name, ' ', &total);
			    for(i = 0; (p = list[i]) != NULL; i++){
				r = strtol(p, &p, 10);
				if(p && *p){
				   fprintf(stderr,"Error at %s\n", list[i]);
				   err++;
				   break;
				}
			    }
			    if(err == 0){
				result = (RESULT_S *) fs_get(sizeof(RESULT_S));
				rp = (int *) fs_get((total + 1)*sizeof(int));
				for(i = 0; (p = list[i]) != NULL; i++)
				   rp[i] = strtol(p, &p, 10);
				rp[i] = 0;
				result->result = (void *) rp;
				result->total  = total;
				vp->result = (void *)result;
			    }
			    for(i = 0; list[i] != NULL; i++)
				fs_give((void **)&list[i]);
			     fs_give((void **)list);
			    break;

	case STRING_TYPE: vp->result = vp->name && *vp->name ? 
					(void *) cpystr(vp->name) : NULL;
		 default: break;
   }

   if(err)
	fprintf(stderr, "Error in input of \"%s\"\n", err_msg);

   return err;
}


void
read_value(char *prompt, GLSDV_S *rvp)
{
  int n;

  rvp->name = fs_get(BUFLEN*sizeof(char));
  for(n = 1; n == 1; ){
    printf(prompt);
    fflush(NULL);
    n = read(0, rvp->name, BUFLEN);
    (rvp->name)[n-1] = '\0';
    if(n == 1 && rvp->defval){
	strcpy(rvp->name, rvp->defval);
	n = 0;	/* exit the loop */
    }
  }
}

/* jacket to read_value */

void
read_input_var(int n)
{
  int rv = 0;
  do {
     if (rv > 0 || glsdv[n].varname.name == NULL)
	read_value(glsdv[n].prompt, &glsdv[n].varname);
     rv = print_error(&glsdv[n].varname, glsdv[n].error_input_type, 
					glsdv[n].varname.type);
     if(rv == 0	&& (glsdv[n].legal)(glsdv[n].varname)){
	fprintf(stderr, glsdv[n].error);
	rv++;
     }
  } while (rv > 0);
}

/* jacket to read_input_var */

void
read_input(GLSDVARS_S **glsdvp)
{
  int i;
  GLSDVARS_S *glsd = *glsdvp;

  for(i = 0; glsd[i].type != LAST_VAR; i++)
    if(glsd[i].type != DUMMY_VAR)
      read_input_var(i);
}

void
copy_args(int argc, char **argv)
{
  int i;

  for (i = 0; glsdv[i].type != LAST_VAR; i++)
    if(glsdv[i].type != DUMMY_VAR)
      glsdv[i].varname.name = argc >= i + 2 ? cpystr(argv[i + 1]) : NULL;
}

char *
double2string(double iv, int precision)
{
  int j,k;
  char rv[BUFLEN], format[BUFLEN];

  for(k = 1, j = 1; iv > k ; k *= 10, j++);
  sprintf(format, "%c%d.%df", '%', j+precision-1, precision-1);
  sprintf(rv, format, iv);
  return cpystr(rv);
}

void
print_result(char *var_name, double iv, int precision)
{
  char *number = double2string(iv, precision);
  printf("%s = %s\n", var_name, number);
  fs_give((void **)&number);
}

void
method_link(GLSDMETHOD_S *method)
{
  GLSDMETHOD_S **rv = &glsd_methods;
  for(; *rv; rv = &((*rv)->next));
  *rv = method;
}

GLSDMETHOD_S *
determine_method(char *argv)
{
  GLSDMETHOD_S *method;
  for(method = glsd_methods; method && ((method->glsd_pick)(argv) == NULL); 
						method = method->next);
  return method;
}

void
add_string(char **result, char *text)
{
  static size_t buflen;
  
  if(!text || !*text)
     return;
  
  if(*result == NULL || **result == '\0'){
     buflen = strlen(text) + 1;
     *result = cpystr(text);
     return;
  }     

  if(strlen(*result) + strlen(text) >= buflen){
     buflen += strlen(text) + 1024;
     fs_resize((void **)result, (size_t) buflen*sizeof(char));
  }
  strcat(*result, text);
}

void *
read_using_method(GLSDMETHOD_S *input_method, GLSDVARS_S **glsdvp)
{
   void *rv;
   (input_method->glsd_init)(glsdvp);
   read_input(glsdvp);
   rv = (input_method->glsd_exec)(*glsdvp, 0);
   (input_method->glsd_free)(NULL);
   return rv;
}

/* this function returns a method. method is always not NULL */

GLSDMETHOD_S *
enter_method(int *argc, char **argv)
{
  GLSDMETHOD_S *method;

  static int first_time = 1;

  if (first_time == 1){
	first_time++;
        method = determine_method(*argc >= 2 ? argv[1] : NULL);
	if(method == NULL)
	  *argc = -1;
  }
  else
     method = (GLSDMETHOD_S *)read_using_method(&read_method_global, &glsdv);

  if (method == NULL){
     fprintf(stderr, "You must select a method to compute!\n");
     method = enter_method(argc, argv);
  }
  return method;
}


int main(int argc, char *argv[])
{
  int done = 0;
  GLSDMETHOD_S *method;
  void *result;

  method_link(&help_method_global);

/* methods to compute using statistical distributions */
  method_link(&lsd_global);

/* Statistical Distributions */
  method_link(&fisher_global);
  method_link(&student_global);
  method_link(&chisquare_global);
  method_link(&normal_global);
  method_link(&fishercv_global);
  method_link(&studentcv_global);
  method_link(&chisquarecv_global);
  method_link(&normalcv_global);

/* Tables of distributions */
  method_link(&tablefisher_global);
  method_link(&tablet_global);
  method_link(&tablechi_global);
  method_link(&tablen_global);
  method_link(&tablecfisher_global);
  method_link(&tablect_global);
  method_link(&tablecchi_global);
  method_link(&tablecn_global);

  method_link(&quit_global);

/* link a method in this list. Also declare it in glsd.h */
/* method_link(&dummy_global); */

  do{
    method = enter_method(&argc, argv);
    if(method->glsd_init)
	(method->glsd_init)(&glsdv);
    if(argc > 0 && method->glsd_copy)	/* if first time around */
	(method->glsd_copy)(argc, argv);
    if(method->glsd_init && method->glsd_copy)
	read_input(&glsdv);
    if(method->glsd_exec){
      result = (method->glsd_exec)(glsdv, DEFAULT_PRECISION);
      (method->glsd_print)(result);
      (method->glsd_free)(&result);
    }
    if(argc ==  method->args || method == &quit_global)
      done = 1;
    argc = 0;	/* avoid accidental exit */
  } while (done == 0);

  exit(0);
}
