#ifndef GLSD_INCLUDED
#define GLSD_INCLUDED
/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 */
#include "mem.h"

#define TT_VAR	0
#define DF_VAR	1
#define DF2_VAR	2
#define P_VAR	3
#define MSE_VAR	4
#define NPG_VAR	5
#define RM_VAR  6
#define BOT_VAR	7
#define TOP_VAR	8
#define STP_VAR	9
#define DUMMY_VAR -1
#define LAST_VAR -2

typedef struct result_t {
  void *result;
  int  total;
} RESULT_S;

typedef struct lsd_var_s {
  char *name;		/* value in string form 	      		*/
  void *result;		/* value transformed to a numeric form 		*/
  int type;		/* type of transformation from name to result	*/
  char *defval;		/* default value				*/
} GLSDV_S;

typedef struct glsdvars {
   int type;
   char *prompt;
   char *error_input_type;
   GLSDV_S varname;
   int (*legal)();
   char *error;
} GLSDVARS_S;


typedef struct glsd_method_s {
  char *name;				/* Name of the method	*/
  char *clswitch;			/* command line switch  */
  int	args;				/* number of args	*/
  struct glsd_method_s *(*glsd_pick)(char *argv);
					/* use this method */
  void (*glsd_init) (GLSDVARS_S **glsdv); 
					/* init method - variables to read */
  void (*glsd_copy) (int argc, char **argv);
					/* load data to method */
  void *(*glsd_exec) (GLSDVARS_S  *glsdv, int precision);	
					/* process data */
  void (*glsd_print) (void *result);
					/* print result */
  void (*glsd_free) (void **result);
					/* free free set them free */
  void *(*glsd_spare) (void *data);	/* spare function */
  struct glsd_method_s *next;		/* next method  */
} GLSDMETHOD_S;


/* Methods known by glsd */

extern GLSDMETHOD_S help_method_global;
extern GLSDMETHOD_S lsd_global;
extern GLSDMETHOD_S student_global;
extern GLSDMETHOD_S studentcv_global;
extern GLSDMETHOD_S fisher_global;
extern GLSDMETHOD_S fishercv_global;
extern GLSDMETHOD_S normal_global;
extern GLSDMETHOD_S normalcv_global;
extern GLSDMETHOD_S chisquare_global;
extern GLSDMETHOD_S chisquarecv_global;
extern GLSDMETHOD_S tablefisher_global;
extern GLSDMETHOD_S tablecfisher_global;
extern GLSDMETHOD_S tablechi_global;
extern GLSDMETHOD_S tablecchi_global;
extern GLSDMETHOD_S tablet_global;
extern GLSDMETHOD_S tablect_global;
extern GLSDMETHOD_S tablecn_global;
extern GLSDMETHOD_S tablen_global;
extern GLSDMETHOD_S quit_global;
/* exterm GLSDMETHOD_S dummy_global */

/* these are not methods to compute, but an easy way to read input,
   consistent with all methods - DO NOT LINK THESE METHODS, THEY DO NOT WORK.
 */
extern GLSDMETHOD_S read_method_global;
extern GLSDMETHOD_S read_answer_global;

/* A global variable needed by read methods */
extern GLSDMETHOD_S *glsd_methods;


#define BUFLEN 1024
#define DEFAULT_PRECISION 8


#define DOUBLE_TYPE		0
#define INTEGER_TYPE		1
#define DOUBLE_LIST_TYPE	2
#define INTEGER_LIST_TYPE	3
#define STRING_TYPE		4

typedef struct glsd_result_t {
  union{
     double dval;
     int    ival;
     char  *sval;
  };
  int type;
  int precision;
} GLSDRESULT_S;

#define NUM_VAR(I,F, N){						 \
 for ((I) = 0; (F)[(I)].type != LAST_VAR && (F)[(I)].type != (N); (I)++);\
    }

GLSDMETHOD_S *determine_method(char *argv);
void copy_args(int argc, char **argv);
void print_result(char *var_name, double iv, int precision);
void add_string(char **result, char *text);
void *read_using_method(GLSDMETHOD_S *input_method, GLSDVARS_S **glsdvp);

#endif /* GLSD_INCLUDED */
