/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 */
#include <stdio.h>
#include <string.h>
#include "glsd.h"
#include "math.h"
#include "lsd.h"
#include "condition.h"

static GLSDVARS_S lsdv []= {
  {DUMMY_VAR, NULL, NULL,
	{NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {TT_VAR, "One or two tailed test : ", "tailed type test",
	{NULL, NULL, INTEGER_TYPE, NULL}, tail_condition, ERROR_TYPE},
  {DF_VAR, "Enter df value         : ", "df",
	{NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {P_VAR,  "Enter P  value         : ", "P",
	{NULL, NULL, DOUBLE_TYPE, NULL}, p_condition, ERROR_P},
  {MSE_VAR, "Enter MSE value        : ", "mse",
	{NULL, NULL, DOUBLE_TYPE, NULL}, mse_condition, ERROR_MSE},
  {NPG_VAR, "Enter N per group      : ", "N",
	{NULL, NULL, INTEGER_TYPE, NULL}, npg_condition, ERROR_NPG},
  {LAST_VAR, NULL, NULL,  
	{NULL, NULL, 0, NULL}, null_condition, NULL}
};

GLSDMETHOD_S lsd_global = {
  "compute lsd",		/* name, used in output of results	*/
  "-lsd",			/* command line switch for this method	*/
   7,				/* number of args from command line	*/
   lsd_pick,			/* pick me, pick me!			*/
   glsdv_init,			/* init method - variables to read 	*/
   copy_args,			/* load data to method			*/
   compute_lsd,			/* process data				*/
   lsd_print,			/* print result				*/
   lsd_free,			/* free free, set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL 	/* next method				*/
};


GLSDMETHOD_S *lsd_pick (char *argv)
{
  GLSDMETHOD_S *rv;
  rv = argv == NULL || !strcmp(argv, lsd_global.clswitch) 
						? &lsd_global : NULL;
  return rv;
}


void
glsdv_init (GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)lsdv;
}

void *
compute_lsd(GLSDVARS_S *lsdv, int precision)
{
  GLSDRESULT_S *result;
  double mse, p, lsd, tv;
  int  tt, df, npg, i;

  NUM_VAR(i,  lsdv, TT_VAR);
  tt = *(int *) ((RESULT_S *)lsdv[i].varname.result)->result;

  NUM_VAR(i,  lsdv, DF_VAR);
  df = *(int *) ((RESULT_S *)lsdv[i].varname.result)->result;

  NUM_VAR(i,  lsdv, P_VAR);
  p = *(double *) ((RESULT_S *)lsdv[i].varname.result)->result;

  NUM_VAR(i,  lsdv, MSE_VAR);
  mse = *(double *) ((RESULT_S *)lsdv[i].varname.result)->result;

  NUM_VAR(i,  lsdv, NPG_VAR);
  npg = *(int *) ((RESULT_S *)lsdv[i].varname.result)->result;

  tv = solve_t(1.0-p/tt, -M_PI/2, M_PI/2, df, precision);
  printf("tv = %1.8f\n", tv); /* Hack for Roberto */
  lsd = tv*sqrt(2*mse/npg);
  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = lsd;
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *) result;
}

void
lsd_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("lsd", rslt->dval, rslt->precision);
}

void
lsd_free(void **result)
{
  int i;

  fs_give(result);
  for(i = 0; lsdv[i].type != LAST_VAR; i++)
     if(lsdv[i].varname.name && *lsdv[i].varname.name){
        fs_give((void **)&lsdv[i].varname.name);
	fs_give(&((RESULT_S *)lsdv[i].varname.result)->result);
        fs_give(&lsdv[i].varname.result);
     }
}
