/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * Math Computations for normal distribution.
 */

#include <string.h>
#include <stdlib.h>
#include "glsd.h"
#include "normal.h"
#include "condition.h"
#include "math.h"


/* Code that computes critical values of the normal distribution */
static GLSDVARS_S normalcv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {TT_VAR, "One or two tailed value : ", "tailed type test",
	{NULL, NULL, INTEGER_TYPE, NULL}, tail_condition, ERROR_TYPE},
  {P_VAR,  "Enter P  value          : ", "P",
        {NULL, NULL, DOUBLE_TYPE, NULL}, p_condition, ERROR_P},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S normalcv_global = {
  "output critical values of the normal distribution",
				/* name, used in output of results	*/
  "-cZ",			/* command line switch for this method	*/
   4,				/* number of args from command line	*/
   normalcv_pick,		/* pick me, pick me!			*/
   normalcv_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   critical_normal,		/* process data				*/
   normalcv_print,		/* print result				*/
   normalcv_free,		/* free free set them free		*/
   NULL, 			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
normalcv_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, normalcv_global.clswitch)) 
					? &normalcv_global : NULL;
  return rv;
}

void
normalcv_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)normalcv;
}  

void *
critical_normal(GLSDVARS_S *stdv, int precision)
{
  GLSDRESULT_S *result;
  double p;
  int i, tt;

  NUM_VAR(i,  stdv, TT_VAR);
  tt = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, P_VAR);
  p = *(double *) ((RESULT_S *)stdv[i].varname.result)->result;

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = solve_normal(1.0-p/tt, -6, 6, precision);
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *)result;
}

void
normalcv_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("Critical normal", rslt->dval, rslt->precision);
}

void
normalcv_free(void **result)
{
  int i;
  fs_give(result);
  for(i = 0; normalcv[i].type != LAST_VAR; i++)
     if(normalcv[i].varname.name && *normalcv[i].varname.name){
        fs_give((void **)&normalcv[i].varname.name);
	fs_give(&((RESULT_S *)normalcv[i].varname.result)->result);
	fs_give(&normalcv[i].varname.result);
     }  
}


/* Code that computes values of the normal distribution */
static GLSDVARS_S normalv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {P_VAR,  "Enter Z value           : ", "Z",
        {NULL, NULL, DOUBLE_TYPE, NULL}, null_condition, NULL},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S normal_global = {
  "output values of the normal distribution",
				/* name, used in output of results	 */
  "-Z",				/* command line switch for this method	*/
   3,				/* number of args from command line	*/
   normal_pick,			/* pick me, pick me!			*/
   normal_init,			/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_normal,		/* process data				*/
   normal_print,		/* print result				*/
   normal_free,			/* free free set them free		*/
   NULL, 			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
normal_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, normal_global.clswitch)) 
					? &normal_global : NULL;
  return rv;
}

void
normal_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)normalv;
}  

void *
compute_normal(GLSDVARS_S *stdv, int precision)
{
  GLSDRESULT_S *result;
  double p;
  int i;

  NUM_VAR(i,  stdv, P_VAR);
  p = *(double *) ((RESULT_S *)stdv[i].varname.result)->result;

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = cumulative_Z(p);
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *)result;
}

void
normal_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("Area for normal", rslt->dval, rslt->precision);
}

void
normal_free(void **result)
{
  int i;
  fs_give(result);
  for(i = 0; normalv[i].type != LAST_VAR; i++)
     if(normalv[i].varname.name && *normalv[i].varname.name){
        fs_give((void **)&normalv[i].varname.name);
	fs_give(&((RESULT_S *)normalv[i].varname.result)->result);
	fs_give(&normalv[i].varname.result);
     }  
}
