/* Test modules for GLSD */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "frac.h"

void simplify_frac(FRAC_S *f)
{
  if(strlen(f->n) > 1 && f->n[strlen(f->n) - 1] == '0' && strlen(f->d) > 1 && f->d[strlen(f->d) - 1] == '0'){
     f->n[strlen(f->n) - 1] = '\0';
     f->d[strlen(f->d) - 1] = '\0';
     simplify_frac(f);
  }
}


int
ourfloor(double x)
{
  return (int) floorf(x);
}


FRAC_S
fltofr_work(double x, int iteration)
{
  int i = -1, j;
  FRAC_S rv, p;

  if(x - ourfloor(x) > 0.99999){
     i = ourfloor(x) + 1;
  } else if(x - ourfloor(x) < 0.00001)
     i = ourfloor(x);

  if(i >= 0){
    rv.s = 1;
    sprintf(rv.n, "%d", i);
    for (i = 1, j = 0; j < iteration; j++)
	i *= 10;
    sprintf(rv.d, "%d", i);
  }
  else{
    p.s = 1;
    sprintf(p.n, "%d", ourfloor(x));
    for (i = 1, j = 0; j < iteration; j++)
	i *= 10;
    sprintf(p.d, "%d", i);
    rv = iteration < 4 ? fracadd(p, fltofr_work(10*(x - ourfloor(x)), iteration + 1)) : p;
  }
  if(iteration == 0)
   rv.s = x >= 0;
  return rv;
}

/* double to frac, accepts x.yz */
FRAC_S
fltofr(double x)
{
  FRAC_S rv;

  rv =  fltofr_work(x, 0);
  return rv;
}

/* double to frac, accepts x.yz */
double
frtofl(FRAC_S x)
{
  char z[DIGITS+1];
  divn(x.n, x.d, z);
  return (x.s == 0 ? -1 : 1)*strtof(z, (char **) NULL);
}


FRAC_S
fracadd(FRAC_S x, FRAC_S y)
{
  unsigned long long i = 1;
  char a[DIGITS+1], b[DIGITS+1];
  FRAC_S z;

  muln(x.d,y.d, z.d);	/* the denominator of the fraction */

  muln(x.n,y.d, a);
  if(x.s == 0){
    padzero(a, 1);
    a[0] = '-';
  }

  muln(y.n,x.d, b);
  if(y.s == 0){
    padzero(b, 1);
    b[0] = '-';
  }

  addn(a,b,z.n);	/* numerator of the fraction */

  z.s = z.n[0] != '-';

  if(z.n[0] == '-'){
   for(i = 1; z.n[i] != '\0'; i++)
     z.n[i-1] = z.n[i];
   z.n[i-1] = '\0';
  }

  return z;  
}

FRAC_S
fracmult(FRAC_S x, FRAC_S y)
{
  FRAC_S t;

  t.s = x.s == y.s;
  muln(x.n, y.n, t.n);
  muln(x.d, y.d, t.d);

  return t;
}
