/* Test module for GLSD */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "int.h"

int decimalposition(char x[DIGITS+1]);
void padzero(char t[DIGITS+1], int n);
void addzero(char t[DIGITS+1], int n);
void shitfdot(char w[DIGITS+1], int total, int wsign, char z[DIGITS+1]);
void adddigit(char a[DIGITS+1], char x[DIGITS+1], int i, int *dot);
void removeleadingzero(char x[DIGITS+1]);

/* auxiliary functions for addn, and such */
void subnumc(char x, char y, NUM_S *r);
void addnumc(char x, char y, NUM_S *r);
void multn1c(char x, char y, NUM_S *r);
void multn1(char x[DIGITS+1], char y, char rv[DIGITS+1]);
char guess_quotient(char a[DIGITS+1], char y[DIGITS+1], int min, int max, char z[DIGITS+1]);

#ifdef DEBUG
int
main(int argc, char **argv)
{
 char x[DIGITS+1], y[DIGITS+1], z[DIGITS+1];
 strcpy(x, argv[1]);
 strcpy(y, argv[2]);
 muln(x, y, z);
 printf("x=%s, y=%s, z=%s\n", x, y, z);
 exit(0);
}
#endif

/* multsavem[0], and multsave[1] are lost memory, can be used as needed */
char multsavem[10][DIGITS+1];
char multsaved[10][DIGITS+1];


void
adddigit(char a[DIGITS+1], char x[DIGITS+1], int i, int *dot)
{
  int n = strlen(a), j = *dot;
  if(strlen(x) <= i){
    j++;
    addzero(a, 1);
  }
  else{
    a[n] = x[i];
    a[n+1] = '\0';
  }
  if(j)
    *dot = j;
}

void
removeleadingzero(char x[DIGITS+1])
{
  int k, i;
  for(k = 0; x[k] == '0'; k++);
  if(k){
    if(x[k] == '.')
       k--;
    for(i = k; x[i] != '\0'; i++)
	x[i-k] = x[i];
    x[i-k] = '\0';
    if(x[0] == '\0')
	strcpy(x, "0");

  }
}


/* returns if x is smaller or equal than y */
int
compare(char x[DIGITS+1], char y[DIGITS+1], int flag)
{
  char a[DIGITS+1], b[DIGITS+1];
  int rv, i, j;

  if(x[0] == '-' && y[0] != '-'){
    rv = flag & CMP_EQL ? 0 : 1;
  } else if (x[0] != '-' && y[0] == '-'){
     rv = 0;
  } else if (x[0] == '-' && y[0] == '-'){
     strcpy(a, x+1);
     strcpy(b, y+1);
     rv = compare(b, a, flag);
  } else{
     strcpy(a, x);
     strcpy(b, y);
     i = decimalposition(a);
     j = decimalposition(b);
     if(i < j)			/* a and b are integers */
	addzero(a, j-i);	/* with the same length */
     else if(i > j)
	addzero(b, i-j);
     i = strlen(a);		/* reuse i and j */
     j = strlen(b);
     if(i < j)
	rv = flag & CMP_EQL ? 0 : 1;
     else{
	if(i > j)
	    rv = 0;
	else{
	  for(i=0; a[i] != '\0' && a[i] == b[i]; i++);
	  rv = (flag & CMP_LEQ) ? i == j || a[i] <= b[i] 
		: (flag & CMP_LES ? i < j && a[i] < b[i] : i == j);
	}
     }
  }
  return rv;
}

/* shift the dot, from the rightmost dot, "total" spaces. Add a sign
  if wsign is equal to 1, and save the answer in z
 */
void
shitfdot(char w[DIGITS+1], int total, int wsign, char z[DIGITS+1])
{
  int i, k,l = strlen(w);

  /* wsign is treated as a shift in the result */
  l = strlen(w);
  z[0] = '-';	/* just in case, may be erased if wsign == 0 */
  if(total < l){ 
	for(k = 0; k < l-total ; k++)
	   z[k+wsign] = w[k];
	z[k+wsign] = '.';
	for(k = l-total; w[k] != '\0'; k++)
	    z[k+1+wsign] = w[k];
	z[total > 0 ? k+1+wsign : k+wsign] = '\0';
  }
  else{
	z[wsign] = '0';
	z[wsign + 1] = '.';
        for (k = 0 ; k < total-l; k++)
	  z[2+k+wsign] = '0';
	for(i=0; w[i] != '\0'; i++)
	  z[2+k+i+wsign] = w[i];
	z[2+k+i+wsign] = '\0';
  }
}

/* count where the decimal is, and eliminate it */
int
decimalposition(char x[DIGITS+1])
{
 int i, k;
  for(i = 0; x[i] != '\0' && x[i] != '.'; i++); 
  if(i < strlen(x)){
    for(k=i; x[k] != '\0' ; k++)
      x[k] = x[k+1];
    x[k] = '\0';
  }
  i = strlen(x) - i;
  return i;
}

/* adds a prefix of zeros, no need to tie off, since the trailing '\0'
   is copied also
 */
void
padzero(char t[DIGITS+1], int n)
{
  int k;
   for(k=strlen(t); k >= 0; k--)
     t[k+n] = t[k];
   for(k=0; k < n; k++)
     t[k] = '0';
}

/* puts "n" zeros at the end of "t" */
void
addzero(char t[DIGITS+1], int n)
{
  int i,j;
  for(i=0; t[i] != '\0'; i++);
  for(j=0; j < n ; j++)
     t[i+j] = '0';
  t[i+n] = '\0';
}

void
subnumc(char x, char y, NUM_S *r)
{
  int subtraction;
  NUM_S rv;

  subtraction = x - y;
  rv.reserve= '0' + (subtraction < 0 ? 1 : 0);
  rv.ldig = '0' + (subtraction < 0 ? 10+subtraction : subtraction);

  if(r->reserve != '0'){
    if(rv.ldig > '0')
       rv.ldig -= 1;
    else{
       rv.ldig = '9';
       rv.reserve = '1';
    }
  }
  r->ldig = rv.ldig;
  r->reserve = rv.reserve;
}


void
addnumc(char x, char y, NUM_S *r)
{
  int addition;
  NUM_S rv;

  addition = (x - '0') + (y - '0');
  rv.reserve= '0' + addition/10;
  rv.ldig = '0' + addition - 10*(addition/10);

  if(r->reserve != '0'){
    if(rv.ldig < '9')
       rv.ldig += 1;
    else{
       rv.ldig = '0';
       rv.reserve = '1';
    }
  }
  r->ldig = rv.ldig;
  r->reserve = rv.reserve;
}


void
multn1c(char x, char y, NUM_S *r)
{
  int mult;
  NUM_S rv, s, t;

  mult = (x - '0')*(y - '0');
  rv.reserve= '0' + mult/10;
  rv.ldig = '0' + mult - 10*(mult/10);

  s.reserve = '0';	/* add without reserve */
  addnumc(rv.ldig, r->reserve, &s);
  r->ldig = s.ldig;
  if(s.reserve != '0'){
     t.reserve = '0';	/* add without reserve */
     addnumc(s.reserve, rv.reserve, &t);
     rv.reserve = t.ldig;
  }
  r->reserve = rv.reserve;
}


void
multn1(char x[DIGITS+1], char y, char rv[DIGITS+1])
{
  int k, l;
  NUM_S r;

  if(y == '0')
    strcpy(rv, "0");
  else if (y == '1')
    strcpy(rv, x);
  else if (multsavem[y-'0'][0] != '\0')
    strcpy(rv, multsavem[y-'0']);
  else{
    l = strlen(x);
    memset((void *)&r, '0', sizeof(NUM_S *));
    for(k = l-1; k>= 0; k--){
      multn1c(x[k],y, &r);
      rv[k+1] = r.ldig;
    }
    rv[0] = r.reserve;
    rv[l+1] = '\0';
    strcpy(multsavem[y-'0'], rv);
  }
} 


void
muln(char x[DIGITS+1], char y[DIGITS+1], char z[DIGITS+1])
{
  int i, j,k,l, sign;
  char w[DIGITS+1], t[DIGITS+1];
  char a[DIGITS+1], b[DIGITS+1];

  /* clear multsavem (last m is for multiplication) */
  for(i = 2; i <= 9; i++)
    multsavem[i][0] = '\0';

  strcpy(w, "0");
 /* save data x and y "as is" */
  strcpy(a, x[0] == '-' ? x+1 : x);
  strcpy(b, y[0] == '-' ? y+1 : y);

  /* sign == 1, means x and y have different sign */
  sign = ((x[0] == '-' ? 1 : 0) + (y[0] == '-' ? 1 : 0)) == 1 ? 1 : 0;

  /* count the position of the decimal, and eliminate it */
  i = decimalposition(a);
  j = decimalposition(b);

  /* now do the multiplication */
  l = strlen(b);
  for(k = l-1; k>=0; k--){
     multn1(a, b[k], t); 
     addzero(t, l-1-k); /* put l-1-k zeros at the end of t 	   */
     addn(t, w, w);	/* add what we got (t), to what we had (w) */ 
  }

  /* and copy it back, including the decimal, if any and the sign*/
  shitfdot(w, i+j, sign, z);
}


/* add decimals x and y. Save addition in z */

void 
addn(char x[DIGITS+1], char y[DIGITS+1], char z[DIGITS+1])
{
 char a[DIGITS+1], b[DIGITS+1], rv[DIGITS+1];
 NUM_S r;
 int i, j, k, l, m, n, sign;

  strcpy(a, x[0] == '-' ? x+1 : x);
  strcpy(b, y[0] == '-' ? y+1 : y);

  /* sign == 1, means x and y have different sign */
  sign = ((x[0] == '-' ? 1 : 0) + (y[0] == '-' ? 1 : 0)) == 1 ? 1 : 0;

  if(sign == 1){	/* a and b have different sign */
    subn(x[0] == '-' ? b : a, x[0] == '-' ? a : b, z);
    return;
  }
  /* else a and b have the same sign, now reuse sign to indicate
    which sign that is */

  sign = x[0] == '-';

  m = decimalposition(a);
  n = decimalposition(b);

  if(m < n)
     addzero(a, n-m);
  else if (m > n)
     addzero(b, m-n);

 /* pad with an initial zero "a" and "b", so we have space for another digit
    in case the sum has a reserve in the last addition.
  */
 padzero(a, 1);
 padzero(b, 1);

 i = strlen(a);
 j = strlen(b);

 if(i < j)
   padzero(a, j-i);
 else if (i > j)
   padzero(b, i-j);

 /* now do the addition */

 r.reserve = '0';
 l = i < j ? j : i;	/* now both a and b have the same length l */
 for (k = l-1; k >= 0; k--){
   addnumc(a[k], b[k], &r);
   rv[k] = r.ldig;
 }
/* rv[0] = r.reserve;*/
 rv[l] = '\0';

 /* now copy the result into z */
 for(k = 0; rv[k] == '0'; k++);
 shitfdot(rv+k, (m<n ? n : m), sign, z);
}


/* subtract x and y, return the result in z */
void
subn(char x[DIGITS+1], char y[DIGITS+1], char z[DIGITS+1])
{
 char a[DIGITS+1], b[DIGITS+1], rv[DIGITS+1];
 NUM_S r;
 int i, j, k, l, m, n, sign;

  if(compare(x,y, CMP_EQL)){ /* easy case! */
    strcpy(z, "0");
    return;
  }

  strcpy(a, x[0] == '-' ? x+1 : x);
  strcpy(b, y[0] == '-' ? y+1 : y);

  /* sign == 1, means x and y have different sign */
  sign = ((x[0] == '-' ? 1 : 0) + (y[0] == '-' ? 1 : 0)) == 1 ? 1 : 0;

  if(sign == 1){	/* x and y have different sign */
    addn(a,b,z);
    if(x[0] == '-'){
	padzero(z,1);
	z[0] = '-';
    }
    return;
  }
  /* else x and y have the same sign. Use sign to indicate the
     sign of the result of x - y */

  sign = compare(x, y, CMP_LEQ);

 /* we will set up the subtraction so that "a" is the bigger and "b" the 
    smaller, regardless of what "a" and "b" where before. */

  if((x[0] == '-' && sign == 0) || (x[0] != '-' && sign == 1)){
     strcpy(a, y+1-sign);
     strcpy(b, x+1-sign);
  }

  m = decimalposition(a);
  n = decimalposition(b);

  if(m < n)
     addzero(a, n-m);
  else if (m > n)
     addzero(b, m-n);

 /* pad with an initial zero "a" and "b", so we have space for another digit
    in case the sum has a reserve in the last addition.
  */
 padzero(a, 1);
 padzero(b, 1);

 i = strlen(a);
 j = strlen(b);

 if(i < j)
   padzero(a, j-i);
 else if (i > j)
   padzero(b, i-j);

 /* now do the subtraction */

 r.reserve = '0';
 l = i < j ? j : i;	/* now both a and b have the same length l */
 for (k = l-1; k >= 0; k--){
   subnumc(a[k], b[k], &r);
   rv[k] = r.ldig;
 }
 rv[l] = '\0';

 /* now copy the result into z */
 for(k = 0; rv[k] == '0'; k++);
 shitfdot(rv+k, (m<n ? n : m), sign, z);
}

char 
guess_quotient(char x[DIGITS+1], char y[DIGITS+1], int min, int max, char z[DIGITS+1])
{
  int guess;
  char b[DIGITS+1], c[DIGITS+1];

  if(min == max)
    return min + '0';

  guess = (min+max)/2;
  if(multsaved[guess][0] == '\0'){
     sprintf(b, "%c", guess + '0');
     muln(y, b, c);
     strcpy(multsaved[guess], c);
  }
  else{
     b[0] = guess + '0';
     if(guess == 0)
	strcpy(c, "0");
     else if (guess == 1)
	strcpy(c, y);
     else strcpy(c, multsaved[guess]);
  }
  if(compare(x, c, CMP_LES)) /* b*y > x, try something lower */
    return guess_quotient(x,y, min, guess, z);

  /* b*y <= x */
  subn(x, c, c);	/* c = x - b*y */
  if(compare(c, y, CMP_LES)){
     strcpy(z, c);
     return b[0];
  }

  if(guess == min){	/* the answer is b[0] = max */
     if(multsaved[max][0] == '\0'){
       sprintf(b, "%c", max + '0');
       muln(y, b, c);
       strcpy(multsaved[max], c);
     }
     else{
	b[0] = max + '0';
        if(guess == 0)
	  strcpy(c, "0");
        else if (guess == 1)
	  strcpy(c, y);
        else strcpy(c, multsaved[max]);
     }
     subn(x, c, c);
     strcpy(z, c);
     return b[0];
  }

  /* we got short, increase b */
  return guess_quotient(x, y, guess, max, z);
}


/* routine that computes the division of two positive integers. The dot
   variable indicates if we have written a "dot" for the answer. The 
   counter iteration starts to be counted when we have set dot to 1.
 */
#define MAXITERATION 100
void 
divwork(char x[DIGITS+1], char y[DIGITS+1], char z[DIGITS+1])
{
  int i, j, dot;
  char a[DIGITS+1];

  /* clear multsaved (d is for division) */
  for (i = 2; i <= 9; i++)
    multsaved[i][0] = '\0';

  a[0] = '\0';
  dot = 0;
  for (i = 0, j = 0; dot < MAXITERATION ; i++){
    if(dot && strcmp(a, "0") == 0)
	break;
    adddigit(a, x, i, &dot);
    removeleadingzero(a);
    if(dot == 1 ){
	if(strcmp(a, "0") == 0)
	   break;
	z[j++] = '.';
    }
    z[j++] = guess_quotient(a, y, 0, 9, a);
  }
  z[j] = '\0';
}

void 
divn(char x[DIGITS+1], char y[DIGITS+1], char z[DIGITS+1])
{
  int i, j, sign;
  char w[DIGITS+1] = {'0'};
  char a[DIGITS+1], b[DIGITS+1];

 /* save data x and y "as is" */
  strcpy(a, x[0] == '-' ? x+1 : x);
  strcpy(b, y[0] == '-' ? y+1 : y);

  /* sign == 1, means x and y have different sign */
  sign = ((x[0] == '-' ? 1 : 0) + (y[0] == '-' ? 1 : 0)) == 1 ? 1 : 0;

  /* count the position of the decimal, and eliminate it */
  i = decimalposition(a);
  j = decimalposition(b);
  removeleadingzero(a);
  removeleadingzero(b);

  divwork(a, b, w);

  removeleadingzero(w);
  strcpy(z, w);
/*  printf("x = %s, y = %s, z = %s\n", x, y, z);*/
  /* and copy it back, including the decimal, if any and the sign
  shitfdot(w, i+j, sign, z); */
}

