/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * sample module
 */
#include <stdio.h>
#include <string.h>
#include "glsd.h"
#include "condition.h"
#include "math.h"
#include "dummy.h"

static GLSDVARS_S dummyv []= {
  {TT_VAR, "Text to ask a question : ", "Text to describe error",
	{NULL, NULL, INTEGER_TYPE}, dummy_condition, DUMMY_ERROR},
  {LAST_VAR, NULL, NULL,  
	{NULL, NULL, 0}, any_condition, NULL}
};

GLSDMETHOD_S dummy_global = {
  "dummy",			/* name, used in output of results	*/
  '-d",				/* command line switch for this method	*/
   2,				/* number of args from command line	*/
   dummy_pick,			/* pick me, pick me!			*/
   dummy_init,			/* init method - variables to read 	*/
   copy_args,			/* load data to method			*/
   compute_dummy,		/* process data				*/
   dummy_print,			/* print result				*/
   dummy_free,			/* free free, set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL 	/* next method				*/
};

GLSDMETHOD_S *dummy_pick(char *argv)
{
  return argv != NULL && !strcmp(argv, dummy_global.clswitch) ? 
				&dummy_global : NULL; 
}

/* error management */

void
dummy_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)dummyv;
}

int
dummy_condition(GLSDV_S dumb)
{
    return(dumb.ival <= 1);
}

void *
compute_dummy(GLSDVARS_S *lsdv, int precision)
{
  GLSDRESULT_S *result;
  int i;
  double p;

  /* get the values of the variables we will use */
  NUM_VAR(i,  lsdv, P_VAR);
  p = *(double *) ((RESULT_S *)dummyv[i].varname.result)->result; 

  /* write the result */
  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->ival  = p*p;	/* This is where the computation is done */
  result->type  = INTEGER_TYPE;
  result->precision = precision;
  return (void *) result;
}

void
lsd_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("dummy", rslt->ival, rslt->precision);
}

void
lsd_free(void **result)
{
  int i;

  fs_give(result);

  for(i = 0; dummyv[i].type != LAST_VAR; i++)
     if(dummyv[i].varname.name && *dummyv[i].varname.name){
        fs_give((void **)&dummyv[i].varname.name);
        fs_give(&((RESULT_S *)dummyv[i].varname.result)->result);
        fs_give(&dummyv[i].varname.result);
     }
}
