/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * Math Computations for glsd.
 */
#include <stdio.h>
#include <string.h>
#include "glsd.h"
#include "student.h"
#include "condition.h"
#include "math.h"

/* code to output critical values of Student's distribution */

static GLSDVARS_S studentcv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {TT_VAR, "One or two tailed value : ", "tailed type test",
        {NULL, NULL, INTEGER_TYPE, NULL}, tail_condition, ERROR_TYPE},
  {DF_VAR, "Enter df value          : ", "df",
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {P_VAR,  "Enter P  value          : ", "P",
        {NULL, NULL, DOUBLE_TYPE, NULL}, p_condition, ERROR_P},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S studentcv_global = {
  "output critical values of Student's distribution",
				/* name, used in output of results	*/
  "-cT",			/* command line switch for this method	*/
   5,				/* number of args from command line	*/
   studentcv_pick,		/* pick me, pick me!			*/
   studentcv_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   critical_t,			/* process data				*/
   studentcv_print,		/* print result				*/
   studentcv_free,		/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
studentcv_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, studentcv_global.clswitch)) 
						? &studentcv_global : NULL;
  return rv;
}

void
studentcv_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)studentcv;
}  

void *
critical_t(GLSDVARS_S *stdv, int precision)
{
  GLSDRESULT_S *result;
  double p, tv;
  int df, tt, i;
 
  NUM_VAR(i,  stdv, TT_VAR);
  tt = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;
  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;
  NUM_VAR(i,  stdv, P_VAR);
  p = *(double *) ((RESULT_S *)stdv[i].varname.result)->result;

  tv = solve_t(1.0-p/tt, -M_PI/2, M_PI/2, df, precision);
  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = tv;
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *)result;
}

void
studentcv_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("Critical T value", rslt->dval, rslt->precision);
}

void
studentcv_free(void **result)
{
  int i;
  fs_give(result);
  for(i = 0; studentcv[i].type != LAST_VAR; i++)
     if(studentcv[i].varname.name && *studentcv[i].varname.name){
        fs_give((void **)&studentcv[i].varname.name);
	fs_give(&((RESULT_S *)studentcv[i].varname.result)->result);
	fs_give(&studentcv[i].varname.result);
     }  
}


/* Code to output values of Student's distribution */


static GLSDVARS_S studentv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR, "Enter df value          : ", "df",
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {P_VAR,  "Enter T  value          : ", "T",
        {NULL, NULL, DOUBLE_TYPE, NULL}, null_condition, NULL},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S student_global = {
  "output values of Student's distribution",
				/* name, used in output of results	*/
  "-T",				/* command line switch for this method	*/
   4,				/* number of args from command line	*/
   student_pick,		/* pick me, pick me!			*/
   student_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_t,			/* process data				*/
   student_print,		/* print result				*/
   student_free,		/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
student_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, student_global.clswitch)) 
						? &student_global : NULL;
  return rv;
}

void
student_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)studentv;
}  

void *
compute_t(GLSDVARS_S *stdv, int precision)
{
  GLSDRESULT_S *result;
  double p;
  int df, i;
 
  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, P_VAR);
  p = *(double *) ((RESULT_S *)stdv[i].varname.result)->result;

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = cumulative_T(p, df);
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *)result;
}

void
student_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("Area value for T", rslt->dval, rslt->precision);
}

void
student_free(void **result)
{
  int i;
  fs_give(result);
  for(i = 0; studentv[i].type != LAST_VAR; i++)
     if(studentv[i].varname.name && *studentv[i].varname.name){
        fs_give((void **)&studentv[i].varname.name);
	fs_give(&((RESULT_S *)studentv[i].varname.result)->result);
	fs_give(&studentv[i].varname.result);
     }  
}

