/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "glsd.h"
#include "condition.h"
#include "math.h"
#include "sort.h"
#include "tableF.h"

/* Code to compute tables of the Fisher distribution */

static GLSDVARS_S tablefisher []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR, "Enter df1                  : ", "df1",
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {DF2_VAR, "Enter df2                  : ", "df2",
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {BOT_VAR,  "Enter Bottom (default 0.0) : ", "bottom number",
        {NULL, NULL, DOUBLE_TYPE, "0.0"}, bottv_condition, ERROR_BOTTV},
  {TOP_VAR,  "Enter Top (no default)     : ", "top number",
        {NULL, NULL, DOUBLE_TYPE, NULL}, toptv_condition, ERROR_TOPTV},
  {STP_VAR,  "Enter Step (default 0.01)  : ", "step",
        {NULL, NULL, DOUBLE_TYPE, "0.01"}, step_condition, ERROR_STEP},   
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S tablefisher_global = {
  "table of values of the Fisher-Snedecor distribution",
				/* name, used in output of results	*/
  "-tF",			/* command line switch for this method  */
   7,				/* number of args from command line	*/
   tablefisher_pick,		/* pick me, pick me!			*/
   tablefisher_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_tablefisher,		/* process data				*/
   tablefisher_print,		/* print result				*/
   tablefisher_free,		/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
tablefisher_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, tablefisher_global.clswitch)) 
						? &tablefisher_global : NULL;
  return rv;
}

void
tablefisher_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)tablefisher;
}  

void *
compute_tablefisher(GLSDVARS_S *stdv, int precision)
{
  int i, j, k, l;
  int df, df2;
  double x, top, bot, step;
  char *tabletext, dummy[BUFLEN];
  GLSDRESULT_S *result;


  tabletext = NULL;

  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, DF2_VAR);
  df2 = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, BOT_VAR);
  bot = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, TOP_VAR);
  top = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, STP_VAR);
  step = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  add_string(&tabletext, "<HTML>\n");
  add_string(&tabletext, "<HEAD>\n");
  sprintf(dummy, "Table of values for the Fisher-Snedecor distribution (df1=%d, df2=%d", df, df2);
  add_string(&tabletext, "<TITLE>");
  add_string(&tabletext, dummy);
  add_string(&tabletext, ")</TITLE>\n");
  add_string(&tabletext, "<style type=\"text/css\">\n");
  add_string(&tabletext, "TD { text-align:center}\n");
  add_string(&tabletext, "TD.header {text-align: center; background-color:#cccccc}\n");
  add_string(&tabletext, "</style>\n");
  add_string(&tabletext, "</HEAD>\n");
  add_string(&tabletext, "<BODY>\n");
  add_string(&tabletext, "<CENTER><H1>Table of values for the Fisher-Snedecor distribution</H1></CENTER>");
  sprintf(dummy, "<CENTER><H2>(df1=%d, df2=%d)</H2></CENTER>\n", df, df2);
  add_string(&tabletext, dummy);
  add_string(&tabletext, "<UL>\n");
  for (l = 1; l == 1 ; l++){
     add_string(&tabletext, "<LI>");
     add_string(&tabletext, "One tailed significance (area between 0 and x)\n");
     add_string(&tabletext, "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=5>\n");
     add_string(&tabletext, "<TR>");
     add_string(&tabletext, "<TD></TD>");
     for(j = 0; j*step < 0.1; j++){
        add_string(&tabletext, "<TD class=header><B>");
	sprintf(dummy, "%1.2f", j*step);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</B></TD>");
     }
     add_string(&tabletext, "</TR>\n");
     for(j = 0; bot + j*0.1 <= top; j++){
        add_string(&tabletext, "<TR>");
        add_string(&tabletext, "<TD class=header><B>");
	sprintf(dummy, "%1.2f", bot + j*0.1);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</B></TD>");
        for(k = 0; k*step < 0.1; k++){
	    add_string(&tabletext, "<TD>");
	    x = bot + j*0.1 + k*step;
	    sprintf(dummy, "%1.5f", cumulative_F(x, df, df2));
	    add_string(&tabletext, dummy);
	    add_string(&tabletext, "</TD>");
	}
        add_string(&tabletext, "</TR>\n");
     }
     add_string(&tabletext, "</TABLE>\n");
     add_string(&tabletext, "<P></P>\n");	/* skip a line */

  }
  add_string(&tabletext, "</UL>\n");
  add_string(&tabletext, "</BODY>\n");
  add_string(&tabletext, "</HTML>\n");

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->sval  = tabletext;
  result->type  = STRING_TYPE;
  return (void *)result;
}

void
tablefisher_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 printf(rslt->sval);
}

void
tablefisher_free(void **result)
{
  int i;

  fs_give((void **) &(*(GLSDRESULT_S **)result)->sval);
  fs_give(result);
  for(i = 0; tablefisher[i].type != LAST_VAR; i++)
     if(tablefisher[i].varname.name && *tablefisher[i].varname.name){
        fs_give((void **)&tablefisher[i].varname.name);
	fs_give(&((RESULT_S *)tablefisher[i].varname.result)->result);
	fs_give(&tablefisher[i].varname.result);
     }
}
/* Code to compute tables of critical values ofthe fisher distribution */

static GLSDVARS_S tablecfisher []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR,  "Enter df1 list         : ", "df1",
        {NULL, NULL, INTEGER_LIST_TYPE, NULL}, dft_condition, ERROR_DFT},
  {DF2_VAR, "Enter df2 list         : ", "df2",
        {NULL, NULL, INTEGER_LIST_TYPE, NULL}, dft_condition, ERROR_DFT},
  {P_VAR,   "Enter P  list          : ", "P",
        {NULL, NULL, DOUBLE_LIST_TYPE, NULL}, pt_condition, ERROR_PT},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S tablecfisher_global = {
  "table of critical values of the Fisher-Snedecor distribution",
				/* name, used in output of results	*/
  "-tcF",			/* command line switch for this method  */
   5,				/* number of args from command line	*/
   tablecfisher_pick,		/* pick me, pick me!			*/
   tablecfisher_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   critical_tablecfisher,	/* process data				*/
   tablecfisher_print,		/* print result				*/
   tablecfisher_free,		/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
tablecfisher_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, tablecfisher_global.clswitch)) 
						? &tablecfisher_global : NULL;
  return rv;
}

void
tablecfisher_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)tablecfisher;
}  

void *
critical_tablecfisher(GLSDVARS_S *stdv, int precision)
{
  int i, j, k;
  int *df, *df2, totaldf, totaldf2, totalp;
  double *p, max;
  char *tabletext, dummy[BUFLEN];
  GLSDRESULT_S *result;


  tabletext = NULL;

  NUM_VAR(i,  stdv, DF_VAR);
  df = (int *) ((RESULT_S *)stdv[i].varname.result)->result;
  totaldf = ((RESULT_S *)stdv[i].varname.result)->total;
  qsort((void *)df, totaldf, sizeof(int), glsd_compare_int);

  NUM_VAR(i,  stdv, DF2_VAR);
  df2 = (int *) ((RESULT_S *)stdv[i].varname.result)->result;
  totaldf2 = ((RESULT_S *)stdv[i].varname.result)->total;
  qsort((void *)df2, totaldf2, sizeof(int), glsd_compare_int);

  NUM_VAR(i,  stdv, P_VAR);
  p = (double *) ((RESULT_S *)stdv[i].varname.result)->result;
  totalp = ((RESULT_S *)stdv[i].varname.result)->total;
  qsort((void *)p, totalp, sizeof(double), glsd_compare_double);

  add_string(&tabletext, "<HTML>\n");
  add_string(&tabletext, "<HEAD>\n");
  add_string(&tabletext, "<TITLE>Table of Critical Values for the Fisher-Snedecor Distribution</TITLE>\n");
  add_string(&tabletext, "<style type=\"text/css\">\n");
  add_string(&tabletext, "TD { text-align:center}\n");
  add_string(&tabletext, "TD.header {text-align: center; background-color:#cccccc}\n");
  add_string(&tabletext, "</style>\n");
  add_string(&tabletext, "</HEAD>\n");
  add_string(&tabletext, "<BODY>\n");
  add_string(&tabletext, "<CENTER><H1>Table of Critical Values for the Fisher-Snedecor distribution</H1></CENTER>\n");
  add_string(&tabletext, "<UL>\n");
  for (k = 0; k < totalp ; k++){
     add_string(&tabletext, "<LI>");
     add_string(&tabletext, "One tailed significance for alpha=");
     sprintf(dummy, "%1.5f", p[k]);
     add_string(&tabletext, dummy);
     add_string(&tabletext, "\n");
     add_string(&tabletext, "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=5>\n");
     add_string(&tabletext, "<TR>");
     add_string(&tabletext, "<TD><B>df2\\df1</B></TD>");
     for(j = 0; j < totaldf; j++){
        add_string(&tabletext, "<TD class=header>");
	sprintf(dummy, "%d", df[j]);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</TD>");
     }
     add_string(&tabletext, "</TR>\n");
     for(i = 0; i < totaldf2; i++){
        add_string(&tabletext, "<TR>");
        add_string(&tabletext, "<TD class=header>");
	sprintf(dummy, "%d", df2[i]);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</TD>");
        for(j = 0; j < totaldf; j++){
	    add_string(&tabletext, "<TD>");
	    for(max = 1.0; cumulative_F(max, df[j], df2[i]) < 1.0 - p[k]; max *=2.0);
	    sprintf(dummy, "%11.9f", solve_fisher(1-p[k], df[j], df2[i], 0, max, 8));;
	    add_string(&tabletext, dummy);
	    add_string(&tabletext, "</TD>");
	}
        add_string(&tabletext, "</TR>\n");
     }
     add_string(&tabletext, "</TABLE>\n");
     add_string(&tabletext, "<P></P>\n");
  }
  add_string(&tabletext, "</UL>\n");
  add_string(&tabletext, "</BODY>\n");
  add_string(&tabletext, "</HTML>\n");

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->sval  = tabletext;
  result->type  = STRING_TYPE;
  return (void *)result;
}

void
tablecfisher_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 printf(rslt->sval);
}

void
tablecfisher_free(void **result)
{
  int i;

  fs_give((void **) &(*(GLSDRESULT_S **)result)->sval);
  fs_give(result);
  for(i = 0; tablecfisher[i].type != LAST_VAR; i++)
     if(tablecfisher[i].varname.name && *tablecfisher[i].varname.name){
        fs_give((void **)&tablecfisher[i].varname.name);
	fs_give(&((RESULT_S *)tablecfisher[i].varname.result)->result);
	fs_give(&tablecfisher[i].varname.result);
     }
}
