/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "glsd.h"
#include "sort.h"
#include "condition.h"
#include "math.h"
#include "tableT.h"

/* Code to compute table of values of Student's distribution */

static GLSDVARS_S tabletv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {TT_VAR, "One or two tailed list     : ", "tailed type test",
        {NULL, NULL, INTEGER_TYPE, NULL}, tailt_condition, ERROR_TYPET},
  {DF_VAR, "Enter df                   : ", "df",
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {BOT_VAR,  "Enter Bottom (default 0.0) : ", "bottom number",
        {NULL, NULL, DOUBLE_TYPE, "0.0"}, bottv_condition, ERROR_BOTTV},
  {TOP_VAR,  "Enter Top (no default)     : ", "top number",
        {NULL, NULL, DOUBLE_TYPE, NULL}, toptv_condition, ERROR_TOPTV},
  {STP_VAR,  "Enter Step (default 0.01)  : ", "step",
        {NULL, NULL, DOUBLE_TYPE, "0.01"}, step_condition, ERROR_STEP},   
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S tablet_global = {
  "table of values of Student's distribution",
				/* name, used in output of results	*/
  "-tT",			/* command line switch for this method  */
   7,				/* number of args from command line	*/
   tablet_pick,			/* pick me, pick me!			*/
   tablet_init,			/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_tablet,		/* process data				*/
   tablet_print,		/* print result				*/
   tablet_free,			/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
tablet_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, tablet_global.clswitch)) 
						? &tablet_global : NULL;
  return rv;
}

void
tablet_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)tabletv;
}  

void *
compute_tablet(GLSDVARS_S *stdv, int precision)
{
  int i, j, k, l, tt, min, max;
  int df;
  double x, top, bot, step;
  char *tabletext, dummy[BUFLEN];
  GLSDRESULT_S *result;


  tabletext = NULL;
  NUM_VAR(i,  stdv, TT_VAR);
  tt = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, BOT_VAR);
  bot = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, TOP_VAR);
  top = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, STP_VAR);
  step = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  min = tt == 1 ? 1 : (tt == 2 ? 2 : 1);
  max = tt == 1 ? 2 : (tt == 2 ? 3 : 3);

  add_string(&tabletext, "<HTML>\n");
  add_string(&tabletext, "<HEAD>\n");
  sprintf(dummy, "%d", df);
  add_string(&tabletext, "<TITLE>Table of Values for T (df=");
  add_string(&tabletext, dummy);
  add_string(&tabletext, ")</TITLE>\n");
  add_string(&tabletext, "<style type=\"text/css\">\n");
  add_string(&tabletext, "TD { text-align:center}\n");
  add_string(&tabletext, "TD.header {text-align: center; background-color:#cccccc}\n");
  add_string(&tabletext, "</style>\n");
  add_string(&tabletext, "</HEAD>\n");
  add_string(&tabletext, "<BODY>\n");
  add_string(&tabletext, "<CENTER><H1>Table of Values for T (df=");
  add_string(&tabletext, dummy);
  add_string(&tabletext, ")</H1></CENTER>\n");
  add_string(&tabletext, "<UL>\n");
  for (l = min; l < max; l++){
     add_string(&tabletext, "<LI>");
     add_string(&tabletext, l == 1 ? "One " : "Two ");
     add_string(&tabletext, "tailed significance");
     add_string(&tabletext, l == 1 ? "(area between -infinity and x)" : "(area between -x and x)");
     add_string(&tabletext, "\n");
     add_string(&tabletext, "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=5>\n");
     add_string(&tabletext, "<TR>");
     add_string(&tabletext, "<TD></TD>");
     for(j = 0; j*step < 0.1; j++){
        add_string(&tabletext, "<TD class=header><B>");
	sprintf(dummy, "%1.2f", j*step);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</B></TD>");
     }
     add_string(&tabletext, "</TR>\n");
     for(j = 0; bot + j*0.1 <= top; j++){
        add_string(&tabletext, "<TR>");
        add_string(&tabletext, "<TD class=header><B>");
	sprintf(dummy, "%1.2f", bot + j*0.1);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</B></TD>");
        for(k = 0; k*step < 0.1; k++){
	    add_string(&tabletext, "<TD>");
	    x = bot + j*0.1 + k*step;
	    sprintf(dummy, "%1.5f", cumulative_T(x, df));
	    add_string(&tabletext, dummy);
	    add_string(&tabletext, "</TD>");
	}
        add_string(&tabletext, "</TR>\n");
     }
     add_string(&tabletext, "</TABLE>\n");
     add_string(&tabletext, "<P></P>\n");	/* skip a line */

  }
  add_string(&tabletext, "</UL>\n");
  add_string(&tabletext, "</BODY>\n");
  add_string(&tabletext, "</HTML>\n");

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->sval  = tabletext;
  result->type  = STRING_TYPE;
  return (void *)result;
}

void
tablet_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 printf(rslt->sval);
}

void
tablet_free(void **result)
{
  int i;

  fs_give((void **) &(*(GLSDRESULT_S **)result)->sval);
  fs_give(result);
  for(i = 0; tabletv[i].type != LAST_VAR; i++)
     if(tabletv[i].varname.name && *tabletv[i].varname.name){
        fs_give((void **)&tabletv[i].varname.name);
	fs_give(&((RESULT_S *)tabletv[i].varname.result)->result);
	fs_give(&tabletv[i].varname.result);
     }
}

/* Code to compute table of critical values of Student's distribution */

static GLSDVARS_S tablectv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {TT_VAR, "One or two tailed list : ", "tailed type test",
        {NULL, NULL, INTEGER_TYPE, NULL}, tailt_condition, ERROR_TYPET},
  {DF_VAR, "Enter df list          : ", "df",
        {NULL, NULL, INTEGER_LIST_TYPE, NULL}, dft_condition, ERROR_DFT},
  {P_VAR,  "Enter P  list          : ", "P",
        {NULL, NULL, DOUBLE_LIST_TYPE, NULL}, pt_condition, ERROR_PT},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S tablect_global = {
  "table of critical values of Student's distribution",
				/* name, used in output of results	*/
  "-tcT",			/* command line switch for this method  */
   5,				/* number of args from command line	*/
   tablect_pick,		/* pick me, pick me!			*/
   tablect_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_tablect,		/* process data				*/
   tablect_print,		/* print result				*/
   tablect_free,		/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
tablect_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, tablect_global.clswitch)) 
						? &tablect_global : NULL;
  return rv;
}

void
tablect_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)tablectv;
}  

void *
compute_tablect(GLSDVARS_S *stdv, int precision)
{
  int i, j, k, tt, min, max;
  int *df, totall, totalp;
  double *p;
  GLSDV_S dft, pt;
  char *tabletext, dummy[BUFLEN];
  GLSDRESULT_S *result;


  tabletext = NULL;
  NUM_VAR(i,  stdv, TT_VAR);
  tt = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, DF_VAR);
  dft = stdv[i].varname;
  df = (int *) ((RESULT_S *)dft.result)->result;
  totall = ((RESULT_S *)dft.result)->total;
  qsort((void *)df, totall, sizeof(int), glsd_compare_int);

  NUM_VAR(i,  stdv, P_VAR);
  pt = stdv[i].varname;
  p = (double *) ((RESULT_S *)pt.result)->result;
  totalp = ((RESULT_S *)pt.result)->total;
  qsort((void *)p, totalp, sizeof(double), glsd_compare_double);

  min = tt == 1 ? 1 : (tt == 2 ? 2 : 1);
  max = tt == 1 ? 2 : (tt == 2 ? 3 : 3);

  add_string(&tabletext, "<HTML>\n");
  add_string(&tabletext, "<HEAD>\n");
  add_string(&tabletext, "<TITLE>Table of Critical Values for T</TITLE>\n");
  add_string(&tabletext, "<style type=\"text/css\">\n");
  add_string(&tabletext, "TD { text-align:center}\n");
  add_string(&tabletext, "TD.header {text-align: center; background-color:#cccccc}\n");
  add_string(&tabletext, "</style>\n");
  add_string(&tabletext, "</HEAD>\n");
  add_string(&tabletext, "<BODY>\n");
  add_string(&tabletext, "<CENTER><H1>Table of Critical Values for T</H1></CENTER>\n");
  add_string(&tabletext, "<UL>\n");
  for (k = min; k < max; k++){
     add_string(&tabletext, "<LI>");
     add_string(&tabletext, k == 1 ? "One " : "Two ");
     add_string(&tabletext, "tailed significance\n");
     add_string(&tabletext, "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=5>\n");
     add_string(&tabletext, "<TR>");
     add_string(&tabletext, "<TD></TD>");
     for(j = 0; j < totalp; j++){
        add_string(&tabletext, "<TD class=header>");
	sprintf(dummy, "%6.5f", p[j]);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</TD>");
     }
     add_string(&tabletext, "</TR>\n");
     for(i = 0; i < totall; i++){
        add_string(&tabletext, "<TR>");
        add_string(&tabletext, "<TD class=header>");
	sprintf(dummy, "%d", df[i]);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</TD>");
        for(j = 0; j < totalp; j++){
	    add_string(&tabletext, "<TD>");
	    sprintf(dummy, "%11.9f", solve_t(1-p[j]/k, -M_PI/2, M_PI/2, df[i], 8));;
	    add_string(&tabletext, dummy);
	    add_string(&tabletext, "</TD>");
	}
        add_string(&tabletext, "</TR>\n");
     }
     add_string(&tabletext, "</TABLE>\n");
  }
  add_string(&tabletext, "</UL>\n");
  add_string(&tabletext, "</BODY>\n");
  add_string(&tabletext, "</HTML>\n");

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->sval  = tabletext;
  result->type  = STRING_TYPE;
  return (void *)result;
}

void
tablect_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 printf(rslt->sval);
}

void
tablect_free(void **result)
{
  int i;

  fs_give((void **) &(*(GLSDRESULT_S **)result)->sval);
  fs_give(result);
  for(i = 0; tablectv[i].type != LAST_VAR; i++)
     if(tablectv[i].varname.name && *tablectv[i].varname.name){
        fs_give((void **)&tablectv[i].varname.name);
	fs_give(&((RESULT_S *)tablectv[i].varname.result)->result);
	fs_give(&tablectv[i].varname.result);
     }
}
