/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "glsd.h"
#include "condition.h"
#include "math.h"
#include "sort.h"
#include "tableX.h"

/* Code to compute tables of the Chi Square distribution */

static GLSDVARS_S tablechi []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR, "Enter df                   : ", "df",
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {BOT_VAR,  "Enter Bottom (default 0.0) : ", "bottom number",
        {NULL, NULL, DOUBLE_TYPE, "0.0"}, bottv_condition, ERROR_BOTTV},
  {TOP_VAR,  "Enter Top (no default)     : ", "top number",
        {NULL, NULL, DOUBLE_TYPE, NULL}, toptv_condition, ERROR_TOPTV},
  {STP_VAR,  "Enter Step (default 0.01)  : ", "step",
        {NULL, NULL, DOUBLE_TYPE, "0.01"}, step_condition, ERROR_STEP},   
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S tablechi_global = {
  "table of values of the Chi Square distribution",
				/* name, used in output of results	*/
  "-tX",			/* command line switch for this method  */
   6,				/* number of args from command line	*/
   tablechi_pick,		/* pick me, pick me!			*/
   tablechi_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_tablechi,		/* process data				*/
   tablechi_print,		/* print result				*/
   tablechi_free,		/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
tablechi_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, tablechi_global.clswitch)) 
						? &tablechi_global : NULL;
  return rv;
}

void
tablechi_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)tablechi;
}  

void *
compute_tablechi(GLSDVARS_S *stdv, int precision)
{
  int i, j, k, l;
  int df;
  double x, top, bot, step;
  char *tabletext, dummy[BUFLEN];
  GLSDRESULT_S *result;


  tabletext = NULL;

  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, BOT_VAR);
  bot = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, TOP_VAR);
  top = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, STP_VAR);
  step = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  add_string(&tabletext, "<HTML>\n");
  add_string(&tabletext, "<HEAD>\n");
  sprintf(dummy, "%d", df);
  add_string(&tabletext, "<TITLE>Table of Values for Chi-Square (df=");
  add_string(&tabletext, dummy);
  add_string(&tabletext, ")</TITLE>\n");
  add_string(&tabletext, "<style type=\"text/css\">\n");
  add_string(&tabletext, "TD { text-align:center}\n");
  add_string(&tabletext, "TD.header {text-align: center; background-color:#cccccc}\n");
  add_string(&tabletext, "</style>\n");
  add_string(&tabletext, "</HEAD>\n");
  add_string(&tabletext, "<BODY>\n");
  add_string(&tabletext, "<CENTER><H1>Table of Values for Chi-Square (df=");
  add_string(&tabletext, dummy);
  add_string(&tabletext, ")</H1></CENTER>\n");
  add_string(&tabletext, "<UL>\n");
  for (l = 1; l == 1 ; l++){
     add_string(&tabletext, "<LI>");
     add_string(&tabletext, "One tailed significance (area between 0 and x)\n");
     add_string(&tabletext, "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=5>\n");
     add_string(&tabletext, "<TR>");
     add_string(&tabletext, "<TD></TD>");
     for(j = 0; j*step < 0.1; j++){
        add_string(&tabletext, "<TD class=header><B>");
	sprintf(dummy, "%1.2f", j*step);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</B></TD>");
     }
     add_string(&tabletext, "</TR>\n");
     for(j = 0; bot + j*0.1 <= top; j++){
        add_string(&tabletext, "<TR>");
        add_string(&tabletext, "<TD class=header><B>");
	sprintf(dummy, "%1.2f", bot + j*0.1);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</B></TD>");
        for(k = 0; k*step < 0.1; k++){
	    add_string(&tabletext, "<TD>");
	    x = bot + j*0.1 + k*step;
	    sprintf(dummy, "%1.5f", cumulative_X(x, df));
	    add_string(&tabletext, dummy);
	    add_string(&tabletext, "</TD>");
	}
        add_string(&tabletext, "</TR>\n");
     }
     add_string(&tabletext, "</TABLE>\n");
     add_string(&tabletext, "<P></P>\n");	/* skip a line */

  }
  add_string(&tabletext, "</UL>\n");
  add_string(&tabletext, "</BODY>\n");
  add_string(&tabletext, "</HTML>\n");

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->sval  = tabletext;
  result->type  = STRING_TYPE;
  return (void *)result;
}

void
tablechi_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 printf(rslt->sval);
}

void
tablechi_free(void **result)
{
  int i;

  fs_give((void **) &(*(GLSDRESULT_S **)result)->sval);
  fs_give(result);
  for(i = 0; tablechi[i].type != LAST_VAR; i++)
     if(tablechi[i].varname.name && *tablechi[i].varname.name){
        fs_give((void **)&tablechi[i].varname.name);
	fs_give(&((RESULT_S *)tablechi[i].varname.result)->result);
	fs_give(&tablechi[i].varname.result);
     }
}
/* Code to compute tables of critical values ofthe Chi Square distribution */

static GLSDVARS_S tablecchi []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR, "Enter df list          : ", "df",
        {NULL, NULL, INTEGER_LIST_TYPE, NULL}, dft_condition, ERROR_DFT},
  {P_VAR,  "Enter P  list          : ", "P",
        {NULL, NULL, DOUBLE_LIST_TYPE, NULL}, pt_condition, ERROR_PT},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S tablecchi_global = {
  "table of critical values of the Chi Square distribution",
				/* name, used in output of results	*/
  "-tcX",			/* command line switch for this method  */
   4,				/* number of args from command line	*/
   tablecchi_pick,		/* pick me, pick me!			*/
   tablecchi_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_tablecchi,		/* process data				*/
   tablecchi_print,		/* print result				*/
   tablecchi_free,		/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
tablecchi_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, tablecchi_global.clswitch)) 
						? &tablecchi_global : NULL;
  return rv;
}

void
tablecchi_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)tablecchi;
}  

void *
compute_tablecchi(GLSDVARS_S *stdv, int precision)
{
  int i, j, k;
  int *df, totall, totalp;
  double *p, max;
  char *tabletext, dummy[BUFLEN];
  GLSDRESULT_S *result;


  tabletext = NULL;

  NUM_VAR(i,  stdv, DF_VAR);
  df = (int *) ((RESULT_S *)stdv[i].varname.result)->result;
  totall = ((RESULT_S *)stdv[i].varname.result)->total;
  qsort((void *)df, totall, sizeof(int), glsd_compare_int);

  NUM_VAR(i,  stdv, P_VAR);
  p = (double *) ((RESULT_S *)stdv[i].varname.result)->result;
  totalp = ((RESULT_S *)stdv[i].varname.result)->total;
  qsort((void *)p, totalp, sizeof(double), glsd_compare_double);

  add_string(&tabletext, "<HTML>\n");
  add_string(&tabletext, "<HEAD>\n");
  add_string(&tabletext, "<TITLE>Table of Critical Values for the Chi-Square Distribution</TITLE>\n");
  add_string(&tabletext, "<style type=\"text/css\">\n");
  add_string(&tabletext, "TD { text-align:center}\n");
  add_string(&tabletext, "TD.header {text-align: center; background-color:#cccccc}\n");
  add_string(&tabletext, "</style>\n");
  add_string(&tabletext, "</HEAD>\n");
  add_string(&tabletext, "<BODY>\n");
  add_string(&tabletext, "<CENTER><H1>Table of Critical Values for the Chi-Square distribution</H1></CENTER>\n");
  add_string(&tabletext, "<UL>\n");
  for (k = 1; k == 1; k++){
     add_string(&tabletext, "<LI>");
     add_string(&tabletext, "One tailed significance\n");
     add_string(&tabletext, "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=5>\n");
     add_string(&tabletext, "<TR>");
     add_string(&tabletext, "<TD><B>df\\p</B></TD>");
     for(j = 0; j < totalp; j++){
        add_string(&tabletext, "<TD class=header>");
	sprintf(dummy, "%6.5f", p[j]);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</TD>");
     }
     add_string(&tabletext, "</TR>\n");
     for(i = 0; i < totall; i++){
        add_string(&tabletext, "<TR>");
        add_string(&tabletext, "<TD class=header>");
	sprintf(dummy, "%d", df[i]);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</TD>");
        for(j = 0; j < totalp; j++){
	    add_string(&tabletext, "<TD>");
	    for(max = 1.0; cumulative_X(max, df[i]) < 1 - p[j]; max *= 2);
	    sprintf(dummy, "%11.9f", solve_chisquare(1-p[j], df[i], 0, max, 8));;
	    add_string(&tabletext, dummy);
	    add_string(&tabletext, "</TD>");
	}
        add_string(&tabletext, "</TR>\n");
     }
     add_string(&tabletext, "</TABLE>\n");
  }
  add_string(&tabletext, "</UL>\n");
  add_string(&tabletext, "</BODY>\n");
  add_string(&tabletext, "</HTML>\n");

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->sval  = tabletext;
  result->type  = STRING_TYPE;
  return (void *)result;
}

void
tablecchi_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 printf(rslt->sval);
}

void
tablecchi_free(void **result)
{
  int i;

  fs_give((void **) &(*(GLSDRESULT_S **)result)->sval);
  fs_give(result);
  for(i = 0; tablecchi[i].type != LAST_VAR; i++)
     if(tablecchi[i].varname.name && *tablecchi[i].varname.name){
        fs_give((void **)&tablecchi[i].varname.name);
	fs_give(&((RESULT_S *)tablecchi[i].varname.result)->result);
	fs_give(&tablecchi[i].varname.result);
     }
}
