/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "glsd.h"
#include "condition.h"
#include "math.h"
#include "sort.h"
#include "tableZ.h"

/* Code for computing tables for the normal distribution */

static GLSDVARS_S tablen []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {TT_VAR, "One or two tailed list     : ", "tailed type test",
        {NULL, NULL, INTEGER_TYPE, NULL}, tailt_condition, ERROR_TYPET},
  {BOT_VAR,  "Enter Bottom (default 0.0) : ", "bottom number",
        {NULL, NULL, DOUBLE_TYPE, "0.0"}, bot_condition, ERROR_BOT},
  {TOP_VAR,  "Enter Top (default 4.0)    : ", "top number",
        {NULL, NULL, DOUBLE_TYPE, "4.0"}, top_condition, ERROR_TOP},
  {STP_VAR,  "Enter Step (default 0.01)  : ", "step",
        {NULL, NULL, DOUBLE_TYPE, "0.01"}, step_condition, ERROR_STEP},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S tablen_global = {
  "table of values of the normal distribution",
				/* name, used in output of results	*/
  "-tZ",			/* command line switch for this method	*/
   6,				/* number of args from command line	*/
   tablen_pick,			/* pick me, pick me!			*/
   tablen_init,			/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_tablen,		/* process data				*/
   tablen_print,		/* print result				*/
   tablen_free,			/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};


GLSDMETHOD_S *
tablen_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, tablen_global.clswitch)) 
				? &tablen_global : NULL;
  return rv;
}

void
tablen_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)tablen;
}  


void *
compute_tablen(GLSDVARS_S *stdv, int precision)
{
  int i, j, k, l, tt, min, max;
  double x, bot, top, step;
  char *tabletext, dummy[BUFLEN];
  GLSDRESULT_S *result;

  tabletext = NULL;
  NUM_VAR(i,  stdv, TT_VAR);
  tt = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, BOT_VAR);
  bot = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, TOP_VAR);
  top = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, STP_VAR);
  step = *(double *)((RESULT_S *)stdv[i].varname.result)->result;

  min = tt == 1 ? 1 : (tt == 2 ? 2 : 1);
  max = tt == 1 ? 2 : (tt == 2 ? 3 : 3);
  
  add_string(&tabletext, "<HTML>\n");
  add_string(&tabletext, "<HEAD>\n");
  add_string(&tabletext, "<TITLE>Table of Values of Z</TITLE>\n");
  add_string(&tabletext, "<style type=\"text/css\">\n");
  add_string(&tabletext, "TD { text-align:center}\n");
  add_string(&tabletext, "TD.header {text-align: center; background-color:#cccccc}\n");
  add_string(&tabletext, "</style>\n");
  add_string(&tabletext, "</HEAD>\n");
  add_string(&tabletext, "<BODY>\n");
  add_string(&tabletext, "<CENTER><H1>Table of Values of Z</H1></CENTER>\n");
  add_string(&tabletext, "<UL>\n");
  for (l = min; l < max; l++){
     add_string(&tabletext, "<LI>");
     add_string(&tabletext, l == 1 ? "One " : "Two ");
     add_string(&tabletext, "tailed significance ");
     add_string(&tabletext, l == 1 ? "(area between -infinity and z)" : "(area between -z and z)");
     add_string(&tabletext, "\n");
     add_string(&tabletext, "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=5>\n");
     add_string(&tabletext, "<TR>");
     add_string(&tabletext, "<TD></TD>");
     for(j = 0; j*step < 0.1; j++){
        add_string(&tabletext, "<TD class=header><B>");
	sprintf(dummy, "%1.2f", j*step);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</B></TD>");
     }
     add_string(&tabletext, "</TR>\n");
     for(j = 0; bot + j*0.1 <= top ; j++){
        add_string(&tabletext, "<TR>");
	add_string(&tabletext, "<TD class=header><B>");
	sprintf(dummy, "%1.2f", bot + j*0.1);
	add_string(&tabletext, dummy);
	add_string(&tabletext, "</B></TD>");
	for(k = 0; k*step < 0.1; k++){
	   add_string(&tabletext, "<TD>");
	   x = bot + j*0.1 + k*step;
	   sprintf(dummy, "%1.5f", l*cumulative_Z(x) + 1 -l);
	   add_string(&tabletext, dummy);
	   add_string(&tabletext, "</TD>");
	}
	add_string(&tabletext, "</TR>\n");
     }
     add_string(&tabletext, "</TABLE>\n");
     add_string(&tabletext, "<P></P>\n");	/* skip a line */
  }
  add_string(&tabletext, "</UL>\n");
  add_string(&tabletext, "</BODY>\n");
  add_string(&tabletext, "</HTML>\n");

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->sval  = tabletext;
  result->type  = STRING_TYPE;
  return (void *)result;
}

void
tablen_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 printf(rslt->sval);
}

void
tablen_free(void **result)
{
  int i;

  fs_give((void **) &(*(GLSDRESULT_S **)result)->sval);
  fs_give(result);
  for(i = 0; tablen[i].type != LAST_VAR; i++)
     if(tablen[i].varname.name && *tablen[i].varname.name){
        fs_give((void **)&tablen[i].varname.name);
	fs_give(&((RESULT_S *)tablen[i].varname.result)->result);
	fs_give(&tablen[i].varname.result);
     }
}

/* Code for computing tables for critical values of the normal distribution */

static GLSDVARS_S tablecn []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {TT_VAR, "One or two tailed list : ", "tailed type test",
        {NULL, NULL, INTEGER_TYPE, NULL}, tailt_condition, ERROR_TYPET},
  {P_VAR,  "Enter P  list          : ", "P",
        {NULL, NULL, DOUBLE_LIST_TYPE, NULL}, pt_condition, ERROR_PT},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S tablecn_global = {
  "table of critical values of the normal distribution",
				/* name, used in output of results	*/
  "-tcZ",			/* command line switch for this method	*/
   4,				/* number of args from command line	*/
   tablecn_pick,		/* pick me, pick me!			*/
   tablecn_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_tablecn,		/* process data				*/
   tablecn_print,		/* print result				*/
   tablecn_free,		/* free free set them free		*/
   NULL,			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};


GLSDMETHOD_S *
tablecn_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, tablecn_global.clswitch)) 
				? &tablecn_global : NULL;
  return rv;
}



void
tablecn_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)tablecn;
}  

void *
compute_tablecn(GLSDVARS_S *stdv, int precision)
{
  int i, j, k, tt, min, max;
  int totalp;
  double *p;
  GLSDV_S pt;
  char *tabletext, dummy[BUFLEN];
  GLSDRESULT_S *result;


  tabletext = NULL;
  NUM_VAR(i,  stdv, TT_VAR);
  tt = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, P_VAR);
  pt = stdv[i].varname;
  p = (double *) ((RESULT_S *)pt.result)->result;
  totalp = ((RESULT_S *)pt.result)->total;

  qsort((void *)p, totalp, sizeof(double), glsd_compare_double);
  min = tt == 1 ? 1 : (tt == 2 ? 2 : 1);
  max = tt == 1 ? 2 : (tt == 2 ? 3 : 3);

  add_string(&tabletext, "<HTML>\n");
  add_string(&tabletext, "<HEAD>\n");
  add_string(&tabletext, "<TITLE>Table of Critical Values of Z</TITLE>\n");
  add_string(&tabletext, "<style type=\"text/css\">\n");
  add_string(&tabletext, "TD { text-align:center}\n");
  add_string(&tabletext, "TD.header {text-align: center; background-color:#cccccc}\n");
  add_string(&tabletext, "</style>\n");
  add_string(&tabletext, "</HEAD>\n");
  add_string(&tabletext, "<BODY>\n");
  add_string(&tabletext, "<CENTER><H1>Table of Critical Values of Z</H1></CENTER>\n");
  add_string(&tabletext, "<UL>\n");
  for (k = min; k < max; k++){
     add_string(&tabletext, "<LI>");
     add_string(&tabletext, k == 1 ? "One " : "Two ");
     add_string(&tabletext, "tailed significance\n");
     add_string(&tabletext, "<TABLE BORDER=0 CELLPADDING=5 CELLSPACING=5>\n");
     add_string(&tabletext, "<TR>");
     for(j = 0; j < totalp; j++){
        add_string(&tabletext, "<TD class=header>");
	sprintf(dummy, "%6.5f", p[j]);
        add_string(&tabletext, dummy);
        add_string(&tabletext, "</TD>");
     }
     add_string(&tabletext, "</TR>\n");
     add_string(&tabletext, "<TR>");
     for(j = 0; j < totalp; j++){
	add_string(&tabletext, "<TD>");
	sprintf(dummy, "%11.9f", solve_normal(1-p[j]/k, -6, 6, 6));;
	add_string(&tabletext, dummy);
	add_string(&tabletext, "</TD>");
     }
     add_string(&tabletext, "</TR>\n");
     add_string(&tabletext, "</TABLE>\n");
  }
  add_string(&tabletext, "</UL>\n");
  add_string(&tabletext, "</BODY>\n");
  add_string(&tabletext, "</HTML>\n");

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->sval  = tabletext;
  result->type  = STRING_TYPE;
  return (void *)result;
}

void
tablecn_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 printf(rslt->sval);
}

void
tablecn_free(void **result)
{
  int i;

  fs_give((void **) &(*(GLSDRESULT_S **)result)->sval);
  fs_give(result);
  for(i = 0; tablecn[i].type != LAST_VAR; i++)
     if(tablecn[i].varname.name && *tablecn[i].varname.name){
        fs_give((void **)&tablecn[i].varname.name);
	fs_give(&((RESULT_S *)tablecn[i].varname.result)->result);
	fs_give(&tablecn[i].varname.result);
     }
}
